@@ -7,15 +7,15 @@
 #include <algorithm> // for std::sort, std::unique, std::accumulate
 #include <unordered_set> // for std::unordered_set
 #include <limits> // for std::numeric_limits
-#include <Rcpp.h>
-
-std::vector<std::vector<int>> nb2vec(Rcpp::List nb);
+#include <cmath> // For std::isnan
 
 std::vector<std::vector<int>> CppLaggedVar4Lattice(std::vector<std::vector<int>> spNeighbor,
                                                    int lagNum);
 
-std::vector<std::vector<double>> GenLatticeEmbeddings(const std::vector<double>& vec,
-                                                      const std::vector<std::vector<int>>& nb,
-                                                      int E);
+std::vector<std::vector<double>> GenLatticeEmbeddings(
+    const std::vector<double>& vec,
+    const std::vector<std::vector<int>>& nb,
+    int E,
+    bool includeself);
 
 #endif // CppLatticeUtils_H
