@@ -11,21 +11,22 @@
 #include "CppStats.h"
 #include "CppLatticeUtils.h"
 #include "SimplexProjection.h"
+#include "SMap.h"
 #include <RcppThread.h>
 
-// Function to compute GCCMSingle4Lattice
 std::vector<std::pair<int, double>> GCCMSingle4Lattice(
     const std::vector<std::vector<double>>& x_vectors,  // Reconstructed state-space (each row is a separate vector/state)
-    const std::vector<double>& y,                      // Time series to be used as the target (should line up with vectors)
-    const std::vector<bool>& lib_indices,              // Vector of T/F values (which states to include when searching for neighbors)
-    int lib_size,                                      // Size of the library
-    int max_lib_size,                                  // Maximum size of the library
-    const std::vector<int>& possible_lib_indices,      // Indices of possible library states
-    const std::vector<bool>& pred_indices,             // Vector of T/F values (which states to predict from)
-    int b                                              // Number of neighbors to use for simplex projection
+    const std::vector<double>& y,                       // Spatial cross-section series to be used as the target (should line up with vectors)
+    const std::vector<bool>& lib_indices,               // Vector of T/F values (which states to include when searching for neighbors)
+    int lib_size,                                       // Size of the library
+    int max_lib_size,                                   // Maximum size of the library
+    const std::vector<int>& possible_lib_indices,       // Indices of possible library states
+    const std::vector<bool>& pred_indices,              // Vector of T/F values (which states to predict from)
+    int b,                                              // Number of neighbors to use for simplex projection
+    bool simplex,                                       // Algorithm used for prediction; Use simplex projection if true, and s-mapping if false
+    double theta                                        // Distance weighting parameter for the local neighbours in the manifold
 );
 
-// Function to compute GCCM4Lattice
 std::vector<std::vector<double>> GCCM4Lattice(
     const std::vector<std::vector<double>>& x_vectors,  // Reconstructed state-space (each row is a separate vector/state)
     const std::vector<double>& y,                       // Spatial cross-section series to cross map to
@@ -35,6 +36,9 @@ std::vector<std::vector<double>> GCCM4Lattice(
     int E,                                              // Number of dimensions for the attractor reconstruction
     int tau,                                            // Spatial lag for the lagged-vector construction
     int b,                                              // Number of nearest neighbors to use for prediction
+    bool simplex,                                       // Algorithm used for prediction; Use simplex projection if true, and s-mapping if false
+    double theta,                                       // Distance weighting parameter for the local neighbours in the manifold
+    int threads,                                        // Number of threads used from the global pool
     bool progressbar = true                             // Whether to print the progress bar
 );
 
